import os
import sqlite3
import subprocess
from sys import exit, prefix
import colorama as col


def get_output(command):
    success = False
    try:
        output = subprocess.check_output(command, stderr=subprocess.STDOUT).decode()
        success = True
    except subprocess.CalledProcessError as e:
        output = e.output.decode()
    except Exception as e:
        # check_call can raise other exceptions, such as FileNotFoundError
        output = str(e)
    return (success, output)


def createDir():
    from appdirs import AppDirs

    dirs = AppDirs("Smartwa", "smartbets_API")
    root = dirs.user_data_dir + "/"
    try:
        if not os.path.isdir(root):
            os.makedirs(root)
    except PermissionError:
        exit(col.Fore.RED + "[*] Run with Sudo PERMISSION!" + col.Fore.RESET)
    except Exception as e:
        pass
    dir = ["database", "log", "text", "analyzed"]
    for data in dir:
        if not os.path.isdir(root + data):
            try:
                os.mkdir(root + data)
            except Exception as e:
                print(f"ERROR : {root+data} - {e}")
    dbnm = root + "database/smartBet2.db"
    return dbnm, root


dbnm, root = createDir()
# Ensures script run with enough permissions
def confirm_sudo():
    try:
        with open(root + "pid", "w") as file:
            file.write(str(os.getpid()))
    except PermissionError:
        exit(col.Fore.RED + "[*] Run with Sudo PERMISSION!" + col.Fore.RESET)
    except Exception as e:
        pass


confirm_sudo()
# Handles db
class database:
    def __init__(self):
        pass

    # Interacts with dbs (sql commands)
    def queryDb(self, sql):
        conn = sqlite3.connect(dbnm)
        csr = conn.cursor()
        try:
            dat = csr.execute(sql)
            data = csr.fetchall()
        except Exception as e:
            data = "0"
        else:
            conn.commit()
            if not data:
                data = "0"
        finally:
            csr.close()
            conn.close()
        return data

    # Inserting data into db[2]
    def insertDb(self, table, col1, col2, data1, data2):
        conn = sqlite3.connect(dbnm)
        csr = conn.cursor()
        run = f"INSERT INTO {table}({col1},{col2}) VALUES(?,?)"
        try:
            csr.execute(run, (data1, data2))
            csr.close()
        except:
            pass
        conn.commit()
        conn.close()
        # Inserting data into db[1]

    def insertDb2(self, table, col, data):
        conn = sqlite3.connect(dbnm)
        csr = conn.cursor()
        run = f"INSERT INTO {table}({col}) VALUES(?)"
        try:
            csr.execute(run, (data))
            csr.close()
        except:
            pass
        conn.commit()
        conn.close()
        # Updating db data

    def updateDb(self, id, column, data, table):
        if type(id) is str:
            id = ["Id", id]
        try:
            conn = sqlite3.connect(dbnm)
            csr = conn.cursor()
            run = (
                f"""UPDATE  {table} SET  {column}='{data}'  WHERE {id[0]}='{id[1]}' """
            )
            csr.execute(run)
            csr.close()
            conn.commit()
            conn.close()
        except Exception as e:
            pass


class set_config:
    def __init__(self, args):
        self.args = args
        self.target = ["log", "color", "filename", "gui", "level"]
        self.db = database()

    # Creates db initially
    def createTable(self):
        self.db.queryDb("DROP TABLE Booter")
        run = f"""CREATE TABLE IF NOT EXISTS Booter(ID INTEGER PRIMARY 
		KEY AUTOINCREMENT, log TEXT, Color TEXT, Filename TEXT, Gui TEXT, Level INTEGER)"""
        self.db.queryDb(run)

    def main(self):
        self.createTable()
        log = self.args.log if self.args.log else False
        color = self.args.color if self.args.color else False
        filename = self.args.filename if self.args.filename else False
        gui = self.args.gui if self.args.gui else False
        level = self.args.level
        self.db.queryDb(
            f"""INSERT INTO Booter(log,color,filename,gui,level)
		VALUES('{log}','{color}','{filename}','{gui}','{level}')"""
        )



class args_handler:
    def __init__(self, **kwargs):
        v = lambda t: kwargs[t]
        self.log = v("log")
        self.color = v("color")
        self.filename = v("filename")
        self.gui = v("gui")
        self.level = v("level")


class predictor:
    def __init__(
        self,
        include_position=False,
        log=False,
        level=0,
        filename=None,
        color=False,
        gui=False,
        api=False,
    ):
        if not api:
            from .configuration_handler import set_config

            args = {
                "log": log,
                "color": color,
                "filename": filename,
                "gui": gui,
                "level": level,
            }
            set_config(args_handler(**args)).main()
        from .bet_common import logging
        from .bet_analyzer import analyze
        from .figure_harvester import harvest

        self.logging, self.harvest, self.analyze = logging, harvest, analyze
        (
            self.position,
            self.res,
            self.A,
            self.B,
            self.aA,
            self.bB,
            self.Af,
            self.Bf,
            self.nF,
        ) = (include_position, "result", [], [], [], [], [], [], {})
        self.zero = {
            "g": 0.0,
            "gg": 0.0,
            "ov15": 0.0,
            "ov25": 0.0,
            "ov35": 0.0,
            "choice": 0.0,
            "result": "0",
            "pick": "0",
        }
        self.recv = (
            {
                "position": 0,
                "win": 0.0,
                "draw": 0.0,
                "loss": 0.0,
                "scored": 0,
                "conceived": 0,
            },
            [0, 0.0, 0.0, 0.0, 0.0],
        )

    # Updates team data in corresponding list
    def update(self, data, opt):
        if opt == 1:
            self.A.clear()
            for val in data:
                self.A.append(val)
        else:
            self.B.clear()
            for val in data:
                self.B.append(val)

    # Exlusively unpacks team-data
    def decide4(self, data, lst):
        pos = ["position", "win", "draw", "loss", "scored", "conceived"]
        for x in range(6):
            pos[x] = data[pos[x]]
        scored, conceived = pos[4], pos[5]
        classA = ((pos[1] + pos[2] + (100 - pos[3])) / 300) * 100
        classB = ((pos[4] + (100 - pos[5])) / 200) * 100
        final = [pos[0], classA, classB]
        self.update(final, lst)

    # Handling results
    def resulter(self):
        two = [self.A[1], self.B[1]]
        self.aA.clear()
        self.bB.clear()
        if max(two) - min(two) <= 10:
            # Adding  goals  in performance deter
            a, b = (sum([self.A[1], self.A[2]]) / 200) * 100, (
                sum([self.B[1], self.B[2]]) / 200
            ) * 100
            self.aA.insert(0, a)
            self.bB.insert(0, b)
        else:
            self.aA.insert(0, self.A[1])
            self.bB.insert(0, self.B[1])
        # Using position in determining result
        if self.position and self.A[0] and self.B[0]:
            self.logging.info(
                f"% of Positions {round(self.A[0],2)}-{round(self.B[0],2)}"
            )
            self.logging.info(f"% Before {round(self.aA[0],2)}-{round(self.bB[0],2)}")
            y = ((self.aA[0] + self.A[0]) / 200) * 100
            z = ((self.bB[0] + self.B[0]) / 200) * 100
            self.aA.insert(0, y)
            self.bB.insert(0, z)
            self.logging.info(f"% After {round(self.aA[0],2)}-{round(self.bB[0],2)}")

    # Equates the two percentages into one
    def evaluater_(self, percs: list) -> list:
        perc_f = lambda v: (v * 100) / sum(percs)
        return [perc_f(percs[0]), perc_f(percs[1])]

    # AI function
    def finalizer(self):
        y, z = self.aA[0], self.bB[0]
        aft = self.evaluater_([y, z])
        if max(aft) - min(aft) <= 10:
            if max(aft) - min(aft) <= 1:
                self.nF[self.res] = "X"
            else:
                if max(aft) == y:
                    self.nF[self.res] = "1X"
                else:
                    self.nF[self.res] = "X2"
        else:
            if y > z:
                self.nF[self.res] = "1"
            else:
                self.nF[self.res] = "2"
        return aft

    # Displays received teams
    def display(self, data, aft):
        if type(data) is list:
            try:
                self.logging.info(data[0].capitalize() + " vs " + data[1].capitalize())
            except:
                pass
        else:
            self.logging.info(data[1].capitalize() + " vs " + data[2].capitalize())
        self.logging.info(f"{round(aft[0],2)} {'*'*5} {round(aft[1],2)}")

    # Computing tertiary predictions (over,GG)
    def analyFn(self, data=0, opt=0):
        finale, all = {}, ["g", "gg", "ov15", "ov25", "ov35"]
        if opt == 1:
            self.Af.extend(data)
        elif opt == 2:
            self.Bf.extend(data)
        else:
            finale.clear()
            for x in range(5):
                finale[all[x]] = (self.Af[x] + self.Bf[x]) / 2
            return finale

    # Determines the final FINAL bet
    def pick(self, dic):
        bef = []
        key = list(dic.keys())
        ch = "__"
        bef.clear()
        t = ["g", "gg", "ov15", "ov25", "ov35", "choice", "result"]
        for x in range(7):
            t[x] = dic[t[x]]
        for x in range(7):
            if x == 6:
                continue
            bef.append(t[x])
            fnl = max(bef)
        for x in range(7):
            if t[x] == fnl:
                ch = key[x]
            if ch == "choice":
                ch = dic["result"]
        return ch

    # Combines the final decision and other predictions
    def render(self, teams):
        self.resulter()
        aft = self.finalizer()
        self.display(teams, aft)
        predi = self.analyFn()
        result = self.nF[self.res]
        scr = [self.aA[0], self.bB[0]]
        rslt = round((max(scr) * 100) / sum(scr), 2)
        predi["choice"] = rslt
        predi[self.res] = result
        predi["pick"] = self.pick(predi)
        return predi

    # Clear global variable's data
    def clearHolders(self):
        self.A.clear
        self.B.clear()
        self.aA.clear()
        self.bB.clear()
        self.Af.clear()
        self.Bf.clear()
        self.nF.clear()
        try:
            del aft
        except:
            pass

    # Handles list data-type
    def predictorL(self, teams: list, net=True) -> dict:
        self.clearHolders()
        for x in range(2):
            mined = self.harvest().harvester(teams[x], net)
            analysis = self.analyze().analyzer(mined)
            if analysis == self.recv:
                return self.zero
            self.decide4(analysis[0], x + 1)
            self.analyFn(analysis[1], x + 1)
        return self.render(teams)

    # Handles dict data-type
    def predictorD(self, teams: dict, net=True) -> dict:
        self.clearHolders()
        for key, value in teams.items():
            mined = self.harvest().harvester(value, net)
            analysis = self.analyze().analyzer(mined)
            if analysis == self.recv:
                return self.zero
            self.decide4(analysis[0], key)
            self.analyFn(analysis[1], key)
        return self.render(teams)



from .bet_common import *
from .html_fetcher import web
import random as rand


class google_net:
    def __init__(self):
        self.one, self.stat, self.done, self.stat2 = [], "/url?q=", [], "/url?esrc"
        self.unwanted = ["women", "u1", "u20", "u21", "u2", "-ii"]

    # Determines integrity of the url
    def verify(self, url):
        ln = len(url)
        if url.endswith("/"):
            if url[ln - 4 : ln - 1].isdigit():
                resp = True
            else:
                resp = False
        elif url[ln - 3 : ln].isdigit():
            resp = True
        else:
            resp = False
        return resp

    # Generates random Google-query params
    def sampl(self):
        web = "soccerway.com"
        key = [
            f"matches in ke.{web}",
            f"in int.{web} website",
            f"in {web}",
            f"football club in ca.{web}",
            f"Performance in {web}",
            f"in {web}",
            f"matches in ke.{web}",
            f"Fixtures in {web}",
        ]
        return rand.choice(key)

    # Handling https link @start
    def httpSearch(self, url):
        if self.verify(url):
            return url
        else:
            return 0

    # Handling  non https links @start
    def statSearch(self, url):
        url1 = url.replace(self.stat, "")
        target = []
        link = ""
        for letter in url1:
            if letter == "&":
                break
            target.append(letter)
        for letter in target:
            link = link + letter
        if self.verify(link):
            return link
        else:
            return 0

    # Launching link-hunt
    def statSearch2(self, url):
        url = url.split("=")
        resp = 0
        for link in url:
            if link[0:5] == "https":
                amp = link.find("&")
                if amp > 0:
                    resp = link[0:amp]
        return resp

    def linkHunter(self):
        for url in self.one:
            if url[0:7] == self.stat:
                ans = self.statSearch(url)
            elif url[0:5] == "https":
                ans = self.httpSearch(url)
            elif url[0:9] == self.stat2:
                ans = self.statSearch2(url)
            if ans:
                self.done.append(ans)

    # Souping 'a' links from raw html
    def first(self, soup):
        links = soup.find_all("a")
        for entry in links:
            url = str(entry.get("href")).strip()
            if ".soccerway.com/teams" in url:
                self.one.append(url)
                for data in self.unwanted:
                    if data in url.lower():
                        try:
                            self.one.remove(url)
                        except:
                            pass

    # Main method
    def glinkFinder(self, team):
        if not "fc" in team.lower():
            team = team + " fc"
        link = google(f"{team} {self.sampl()}")
        soup = souper(web().fetcher(link, 0))
        self.first(soup)
        self.linkHunter()
        return self.done



import datetime, sqlite3, os, time, re, traceback, sys, calendar, subprocess
from bs4 import BeautifulSoup as bts
from .configuration_handler import dbnm, root
from random import randint
import logging as log
import colorama as col
import threading as thr
import colorama as col

version = 1.1
config = {}
env = {"api": False}
now, ht = datetime.datetime.today(), "text/"
timeNow = now.strftime("%H:%M:%S")
timeStamp = now.strftime("%d_%b_%Y-%H_%M_%S.")
tablee = now.strftime("%b_%d_%Y")
tbe = "Bet_" + tablee
predictionTb = "Prediction_" + tablee
developer = "Smartwa"
root_image = os.getcwd() + "static/image/"
logo = root_image + "logo.jpg"
predbets_img = root_image + "predbets.png"
# Runs system cmds at API level
def get_output(command):
    success = False
    try:
        output = subprocess.check_output(command, stderr=subprocess.STDOUT).decode()
        success = True
    except subprocess.CalledProcessError as e:
        output = e.output.decode()
    except Exception as e:
        # check_call can raise other exceptions, such as FileNotFoundError
        output = str(e)
    return (success, output)


# Gets environment [tar termux]
def environ_setup():
    if "termux" in str(sys.prefix).lower():
        env["api"] = True
        """
		if 'version' and 'maintainer'  in str(get_output('pkg show termux-api')[1]).lower():
			env['api']=True
		else:
			if not 'pydroid' in str(sys.prefix).lower():
				opt=input('Install Termux-API.  [Y/n]:')
				if opt.lower()=='y':
					os.system('pkg install termux-api')
					print('Termux-api will function on next run!')
		"""


environ_setup()


def filter_str(fnm, sp="_"):
    un = [t for t in str(fnm) if not t.isalnum() and not t in list(sp)]
    for val in un:
        fnm = str(fnm).replace(val, "")
    return str(fnm)


# Gets latest time
def c_time():
    now = datetime.datetime.today()
    rp = now.strftime("%d-%b-%Y %H:%M:%S")
    return rp


def correctDt(no):
    if not str(no)[0] == "0" and int(no) < 10:
        no = "0" + str(no)
    return no


def cent():
    tm = sys.prefix
    if "termux" in tm:
        return 45
    else:
        return 37


def tableDay(day="none", month=False, yr=now.year):
    if str(day).isdigit() and not month:
        no = correctDt(day)
        no = now.strftime(f"%b_{no}_{yr}")
    elif str(day).isdigit() and str(month).isdigit():
        no = correctDt(day)
        no = now.strftime(f"{calendar.month_abbr[int(month)]}_{no}_{yr}")
    else:
        no = tablee
    return no


def showProgress(value, total, msg="Processing"):
    dev = 0
    pre = sys.prefix
    tm = 13
    if "termux" in str(pre):
        tm = 20
    resp = f"•{msg}-{'#'*tm}~[{round(((value*100)/total)+dev,2)} %] "
    return resp


# Converts datatype to list
def convToList(data):
    file8 = []
    for dat in data:
        file8 = file8 + list(dat)
    return file8


# Filters out non-alnum x-ters from string
def filter(link, fmt=""):
    link = str(link)
    for data in link:
        if not data.isalnum() and data != "_":
            link = link.replace(data, "")
    return link + "." + fmt


# Running sql statements against db
def queryDb(sql):
    conn = sqlite3.connect(dbnm)
    csr = conn.cursor()
    try:
        dat = csr.execute(sql)
        data = csr.fetchall()
    except:
        data = "0"
    else:
        conn.commit()
        if not data:
            data = "0"
    finally:
        csr.close()
        conn.close()
    return data


def sample4(a=10000, b=99999):
    return " " + str(randint(a, b))


# Inserting data into db[2]
def insertDb(table, col1, col2, data1, data2):
    tm4 = c_time() + sample4()
    conn = sqlite3.connect(dbnm)
    csr = conn.cursor()
    run = f"INSERT INTO {table}({col1},{col2},Creation_time) VALUES(?,?,?)"
    try:
        csr.execute(run, (data1, data2, tm4))
        csr.close()
        conn.commit()
        conn.close()
    except:
        pass
    return tm4


# Inserting data into db[1]
def insertDb2(table, col, data):
    conn = sqlite3.connect(dbnm)
    csr = conn.cursor()
    run = f"INSERT INTO {table}({col}) VALUES(?)"
    try:
        csr.execute(run, (data))
        csr.close()
        conn.commit()
        conn.close()
    except:
        pass


# Updating db data
def updateDb(id, column, data, table):
    if type(id) is str:
        id = ["Id", id]
    try:
        conn = sqlite3.connect(dbnm)
        csr = conn.cursor()
        run = f"""UPDATE  {table} SET  {column}='{data}'  WHERE {id[0]}='{id[1]}' """
        csr.execute(run)
        csr.close()
        conn.commit()
        conn.close()
    except:
        pass


# Soups data
def souper(html, prett=False):
    nm = bts(str(html), "html.parser")
    if prett:
        nm = nm.prettify()
    return nm


# Filters out irrelevant tags
def soupHtml(data, amt=2, sp="", pret=0):
    soup = souper(data)
    if int(amt) == 1:
        lst = ["script"]
    else:
        lst = ["script", "style"]
    for data in soup(lst):
        data.decompose
    after = f"{sp}".join(soup.stripped_strings)
    if pret:
        soup = souper(after)
        after = soup.prettify()
    return str(after)


# Getting names from Html
def nameFinder(html, sp=""):
    name = soupHtml(html, 2, sp)
    name = name.replace("'", "")
    if str(name).lower() == "none":
        name = "?"
    return name


# Generating google search link
def google(search):
    key = re.sub("\s", "+", search)
    stat = f"""https://www.google.com/search?q={key}"""
    return stat


# Saves text-like files
def saver(data, link, fmt, mode="w", dir=ht):
    fnm = filter(link, fmt)
    data = souper(data, 1)
    with open(root + dir + fnm, mode) as file:
        file.write(data)
        file.close()


# Opens file (txt,html)
def opener(link, fmt="html"):
    try:
        with open(ht + filter(link, fmt)) as file:
            data = file.read()
    except:
        data = 0
    finally:
        return data


def getInt(score):
    score = re.sub("\s", "", score)
    after = []
    score = score.split("-")
    for dat in score:
        for letter in dat:
            if not letter.isdigit():
                dat = dat.replace(letter, "")
        after.append(int(dat))
    return after


# Analysing results
def tScore(score):
    sA, sB = [], []
    for fig in score:
        if not fig:
            fig = "0-0"
        integ = getInt(fig)
        sA.append(min(integ))
        sB.append(max(integ))
    return [sum(sA), sum(sB)]


# Handles the config passed
def run(cmd):
    osr = lambda a: os.system(a)
    try:
        t = thr.Thread(
            target=osr,
            args=(cmd,),
        )
        t.start()
        t.join(timeout=1)
    except:
        pass


# Updates configurations parsed
class booter:
    def __init__(self):
        self.target = ["log", "color", "filename", "gui", "level"]

    def get_info(self):
        info = queryDb(f'SELECT {",".join(self.target)} FROM Booter WHERE ID=1')[0]
        for x in range(len(self.target)):
            if str(info[x]).lower() in ("false", "0", "none"):
                config[self.target[x]] = False
            else:
                config[self.target[x]] = info[x]


booter().get_info()
# Logging configurations
def log_level():
    tg = int(config["level"])
    return tg if tg > 0 else 60


if config.get("log"):
    fnm = config.get("filename")
    if fnm:
        fnm = filter_str(fnm, "._")
        if not "." in fnm:
            fnm = fnm + ".log"
        if "/" in fnm:
            try:
                os.makedirs(os.path.split(fnm[0]))
            except:
                pass
        else:
            fnm = "/log/" + fnm
    else:
        fnm = "/log/" + timeStamp + "log"
    log.basicConfig(
        format="%(asctime)s - %(levelname)s : %(message)s",
        datefmt="%H:%M:%S",
        level=log_level(),
        filename=root + fnm,
    )
else:
    log.basicConfig(
        format="%(asctime)s - %(levelname)s : %(message)s",
        datefmt="%H:%M:%S",
        level=log_level(),
    )
# COLOR CONFIG
bold = col.Style.NORMAL
if config.get("color"):

    class logging:
        def __init__(self):
            self.color = self.color()

        class color:
            grey = "\x1b[38;20m"
            yellow = "\x1b[33;20m"
            red = "\x1b[31;20m"
            bold_red = "\x1b[31;1m"
            reset = "\x1b[0m"
            reset = col.Fore.GREEN

        def debug(self, arg):
            log.debug(bold + self.color.grey + arg + self.color.reset)

        def info(self, arg):
            log.info(bold + col.Fore.CYAN + arg + self.color.reset)

        def warning(self, arg):
            log.warning(bold + self.color.yellow + arg + self.color.reset)

        def error(self, arg):
            log.error(bold + self.color.red + arg + self.color.reset)

        def critical(self, arg):
            log.critical(bold + self.color.bold_red + arg + self.color.reset)

else:

    class logging:
        def __init__(self):
            self.color = self.color()

        class color:
            reset = ""  # col.Fore.GREEN

        def debug(self, arg):
            log.debug(arg + self.color.reset)

        def info(self, arg):
            log.info(arg + self.color.reset)

        def warning(self, arg):
            log.warning(arg + self.color.reset)

        def error(self, arg):
            log.error(arg + self.color.reset)

        def critical(self, arg):
            log.critical(arg + self.color.reset)


logging = logging()
if env["api"] and config.get("gui"):

    class guint:
        def __init__(sel):
            pass

        def toast(self, msg, s="-s", g="bottom", b="darkgray", c="lime"):
            if not s:
                s = ""
            run(f"termux-toast -g {g} -b {b} -c {c} {s} {msg}")

        def notify(self, msg, t="Smartbets", gr=1, id=1, img=logo):
            run(
                f"termux-notification -t {t} -c {'-'.join(msg.split(' '))} --group {gr} --id {id} --image-path {img}"
            )

        def remove(self, id=1110):
            run(f"termux-notification-remove {id}")

        def vibrate(self, d=1, f=""):
            if bool(f):
                f = "-f"
            run(f"termux-vibrate -d {str(d*150)} {f}")

        def speak(
            self,
            msg,
            r=1.0,
            p=1.0,
            s="MUSIC",
        ):
            run(f"termux-tts-speak -p {p} -r {r} -s {s} {msg}")

else:

    class guint:
        def __init__(self):
            pass

        def toast(self, msg, s="-s", g="top", b="gray", c="lime"):
            pass

        def notify(self, msg, t="Smartbets", gr=1, id=1110, img=False):
            pass

        def remove(self, id=1110):
            pass

        def vibrate(self, d=1, f=False):
            pass

        def speak(self, msg, p=1.0, r=1.0, s="MUSIC"):
            pass


# terminates the program
def exit_t(msg=""):
    logging.critical(f"Healthy exit ~ {msg}")
    sys.exit(
        col.Fore.YELLOW
        + f"[*] Goodbye! ^Regards:{col.Fore.GREEN+developer}"
        + col.Fore.RESET
    )


gui = guint()
gui.toast("SmartbetsAPI running", b="red", s=False, c="yellow", g="top")
gui.speak(f"Regards : {developer}")
gui.notify(f"{developer.split(' ')[0]}", "Developer", 1.0, 1.0, logo)



from .bet_common import logging, getInt, tScore
from .tertiary_bet import tertiary
import os
import re


class analyze:
    def __init__(self):
        self.gSco, self.gCon = [], []

    # Converts  goal-scored  to  (%)
    def goaler(self, amt, total):
        try:
            perc = (amt * 100) / total
        except:
            return 0
        return perc

    # Determines total scores (%)
    def nScores(self, amt, over=5):
        perc = (amt * 100) / over
        return perc

    # Determines team position (%)
    def positioner(self, pos, amt=20):
        one = 100 / amt
        before = pos * one
        if before == 0:
            return 0
        return (100 + one) - before

    # Main function (Fig analysis controller)
    def analyzer(self, coded):
        logging.debug(f"Analyzing {len(coded)} item(s)")
        scores = coded[0]
        position = coded[4]
        t = ["score", "win", "draw", "loss", "position"]
        n = ["nWin", "nDraw", "nLoss"]
        for x in range(3):
            n[x] = scores[t[x + 1]]
        for x in range(3):
            t[x + 1] = coded[x + 1]
        wdl = [t[1], t[2], t[3]]
        _pos = self.positioner(position)
        _nWin = self.nScores(n[0])
        _nDraw = self.nScores(n[1])
        _nLoss = self.nScores(n[2])
        # updating goals
        for x in range(1, 4):
            analyzed = tScore(t[x])
            if x == 1 or x == 2:
                self.gSco.append(max(analyzed))
                self.gCon.append(min(analyzed))
            if x == 3:
                self.gSco.append(min(analyzed))
                self.gCon.append(max(analyzed))
        y = sum(self.gSco)
        z = sum(self.gCon)
        gTotal = y + z
        _gSco = self.goaler(y, gTotal)
        _gCon = self.goaler(z, gTotal)
        analysis = {
            "position": _pos,
            "win": _nWin,
            "draw": _nDraw,
            "loss": _nLoss,
            "scored": _gSco,
            "conceived": _gCon,
        }
        tertia = tertiary(wdl)
        return (analysis, tertia)



from .html_fetcher import web
from .bet_common import *
from .googler import google_net


class harvest:
    def __init__(self):
        self.score, self.win, self.draw, self.loss, self.usedName = {}, [], [], [], []

    # Overall performance
    def queryTeams(self, tbl):
        tms = convToList(queryDb(f"SELECT Name FROM {tbl}"))
        return tms

    def getTeamsB(self, tbl, soup=False):
        logging.debug("Updating Teams B")
        one = soup.find_all("div", {"class": "submenu_dropdown"})
        soup1 = souper(one)
        t1 = soup1.find_all("option")
        availableT = self.queryTeams(tbl)
        for line in t1:
            link = line.get("value")
            title = nameFinder(line)
            if link and title and not title in availableT:
                insertDb(f"{tbl}", "Name", "Url", title, link)

    # Getting Home  teams
    def getTeamsA(self, tbl="Team", soup=False):
        logging.debug("Updating Teams A")
        one = soup.find_all("td", {"class": "text team large-link"})
        soup1 = souper(one)
        t1 = soup1.find_all("a")
        availableT = self.queryTeams(tbl)
        for line in t1:
            link = line.get("href")
            title = line.get("title")
            if link and title and not title in availableT:
                insertDb(f"{tbl}", "Name", "Url", title, link)
        self.getTeamsB(tbl, soup=soup)

    # Classifying scores
    def performance(self, soup):
        logging.debug("Team Performance")
        targets = ["win", "draw", "loss"]
        x = 0
        for target in targets:
            a = soup.find_all("a", {"class": f"result-{target}"}, limit=1000)
            self.score[target] = len(a)
            x += 1
            for entry in a:
                result = nameFinder(entry)
                if x == 1:
                    self.win.append(result)
                elif x == 2:
                    self.draw.append(result)
                else:
                    self.loss.append(result)

    # Getting team position
    def position(self, st=0, st1=False, soup=False):
        keys = ["odd", "even"]
        a = soup.find_all("tr", {"class": f"{keys[st]} highlight team_rank"})
        if a:
            b = soupHtml(a[0], sp="\n")
            c = b.split("\n")
            logging.debug(f"Pos-Team-MP-D-P:\n{c}")
            return c[0]
        if st1:
            return str(0)
        else:
            return self.position(1, 2, soup)

    # Establishing dbs
    def createDb(self, tbl="Team"):
        logging.debug("Database Configuration")
        run = f"""CREATE TABLE IF NOT EXISTS {tbl}(
		Id INTEGER PRIMARY KEY AUTOINCREMENT,
		Name TEXT UNIQUE, Url TEXT UNIQUE, Creation_time TEXT)"""
        queryDb(run)

    # Main function
    def lenTeams(self):
        run = "SELECT COUNT(name) FROM league5"
        no = queryDb(run)
        return int(no[0][0])

    # Error detected ***
    def firstTeams(self, net=False):
        logging.warning("Updating league [5]")
        gui.toast("Updating League[5]")
        self.createDb("League5")
        self.createDb()
        top = [
            "int.soccerway.com/teams/england/manchester-city-football-club/676/",
            "int.soccerway.com/teams/spain/club-atletico-de-madrid/2020/",
            "int.soccerway.com/teams/germany/fc-bayern-munchen/961/",
            "int.soccerway.com/teams/italy/fc-internazionale-milano/1244/",
            "int.soccerway.com/teams/france/paris-saint-germain-fc/886/",
        ]
        for link in top:
            befNo = self.lenTeams()
            aftNo = 0
            soup = souper(web().fetcher(link, net))
            logging.info(nameFinder(soup.title))
            while aftNo <= befNo + 14:
                self.getTeamsA("league5")
                self.getTeamsA(soup=soup)
                aftNo = self.lenTeams()
            del soup

    # Verifies link's integrity
    def linkVerify(self, link):
        link2 = link
        x = 0
        while 1:
            if link[(len(link) - 2)].isdigit():
                return link
            elif x > 50 or len(link) < 4:
                return link2
            ind = link.rfind("/")
            link = link[0:ind]
            x += 1

    def harvester(self, name, net=True, linkF=False):
        self.createDb()
        # Handles teams that can't be found on Google
        if len(self.usedName) > 4 and self.usedName[0] == self.usedName[3]:
            axy = ({"win": 0, "draw": 0, "loss": 0}, [], [], [], 0)
            logging.warning(f"No data found for '{name}' ")
            data = f"{timeStamp} : '{name}' Team-info wasn't found on Web!"
            saver(data, link=f"{tbe}_report", fmt="txt", mode="a", dir="text/")
            self.usedName.clear()
            return axy
        if linkF:
            links = convToList(
                queryDb(f"SELECT url FROM Team WHERE url LIKE '%{linkF}' ")
            )
        else:
            links = convToList(
                queryDb(f"SELECT url FROM Team WHERE Name LIKE'%{name}' ")
            )
        if not "0" in links:
            x = 0
            link = links[0]
            if len(link) > 10:
                link = self.linkVerify(link)
                main = "https://ke.soccerway.com"
                if not "https" in link:
                    link = main + link
                soup = souper(web().fetcher(link, net))
                x += 1
                ct = nameFinder(soup.title).split("-")[:2]
                if len(ct) > 1:
                    logging.info(f"{ct[0]} - {ct[1]}")
                self.createDb()
                self.getTeamsA(soup=soup)
                self.performance(soup)
                if len(self.score) == 3:
                    pos = self.position(soup=soup)
                    if not pos.isdigit():
                        pos = 0
                    del soup, links
                    self.usedName.clear()
                    fdb = (self.score, self.win, self.draw, self.loss, int(pos))
                    return fdb
                else:
                    logging.warning(f"{x}.Link :'{link}' failed!")
                    return self.harvester(name, net=True)
        else:
            logging.debug(f"{name} NOT in dbs")
            link = google_net().glinkFinder(name)
            if link:
                logging.debug(f"{name} being Updated")
                insertDb("Team", "Name", "Url", name, link[0])
                return self.harvester(name, net=net, linkF=link[0])
            else:
                logging.debug("Zero G-links Found")
        self.usedName.append(name)
        return self.harvester(name, net=True)



from .bet_common import *
import requests as req
import random as rand
from faker import Faker


class web:
    def __init__(self):
        self.used, self.exc, self.user_agent, self.ref = [], [], [], []

    def sampl(self):
        etc = [
            "int",
            "ke",
            "us",
            "ca",
            "uk",
            "ca",
            "id",
            "cn",
            "de",
            "au",
            "gh",
            "in",
            "ie",
            "my",
            "ng",
            "nr",
            "sg",
            "za",
            "es",
            "el",
            "fr",
            "gr",
            "it",
            "kr",
            "nl",
            "pl",
            "pt",
            "br",
            "ro",
            "th",
            "tr",
            "ru",
            "ar",
        ]
        add = rand.choice(etc)
        if len(self.used) == len(etc):
            return add
        elif add in self.used:
            return self.sampl()
        else:
            if not add in self.used:
                self.used.append(add)
            return add

    def repair(self, url):
        if "soccerway.com/teams/" in url:
            stro = url.find(".")
            ln = len(url)
            return self.sampl() + url[stro:ln]
        return url

    # Counter-checks http status_code
    def status(self, code, reason):
        inf = f"{code} : {reason}"
        if not code == 200:
            logging.warning(f"{code} - {reason}")
        else:
            logging.debug(inf)

    # Generating request headers
    def header(self, user_ag=False):
        if not user_ag:
            user_ag = self.user_agent
        google = "https://www.google.com/"
        if user_ag:
            userA = self.user_agent[0]
            referer = google
            if self.ref:
                referer = self.ref[0]
            setchF = "same-origin"
        else:
            userA = f"{Faker().user_agent()}"
            referer = google
            setchF = "none"
            self.user_agent.insert(0, userA)
        content = {
            "Accept": "text/html, application/xhtml+xml, application/xml",
            "Accept-Language": "en-US,en;q=0.5",
            "Accept-Encoding": "gzip, deflate",
            "Referer": referer,
            "X-Requested-With": "com.opera.mini.native.beta",
            "Connection": "Keep-Alive",
            "User-Agent": userA,
            "Sec-Fetch-Site": setchF,
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Dest": "document",
        }
        return content

    # Getting html from web
    def sender(self, link, info="Requesting", g=0, head=0):
        ht = "http"
        if ht in link:
            url = link
        else:
            url = "https://" + link
        try:
            logging.info(f"{info} - {url}")
            resp = req.get(url=url, headers=self.header())
        except Exception as e:
            if len(self.exc) > 1:
                return "0"
            logging.error(str(e))
            self.exc.append("1")
            return self.sender(url, "Retrying")
        else:
            self.ref.insert(0, url)
            self.status(resp.status_code, resp.reason)
            try:
                content = resp.text
            except:
                pass
            finally:
                code = resp.status_code
                g += 1
                if not code == 200:
                    self.user_agent.clear()
                    gui.toast(f"Http-code {code}", b="red", c="lime")
                    logging.debug(f"{g}s of SLEEP")
                    time.sleep(g)
                    return self.sender(self.repair(url), "Retrying", g, 1)
                if "access denied" in soupHtml(content).lower():
                    logging.warning("ACCESS DENIED".center(37))
                    self.user_agent.clear()
                    logging.warning(f"{g}s of SLEEP")
                    time.sleep(g)
                    return self.sender(self.repair(url), "Retrying", g, 1)
                nm = filter(link, "html")
                queryDb(f"DELETE FROM html where F_name='{nm}' ")
                insertDb("Html", "F_name", "File", nm, content)
                return self.launcher(link)

    # Query link's contents from db
    def launcher(self, link):
        nm = filter(link, "html")
        stat = convToList(queryDb(f"SELECT file FROM html WHERE F_name='{nm}' "))[0]
        if len(stat) > 50:
            return stat
        else:
            return self.sender(link)

    # Initially creating db
    def createDb(self):
        run = """CREATE TABLE IF NOT EXISTS html(Id INTEGER PRIMARY KEY
			AUTOINCREMENT, F_name TEXT, File TEXT, Creation_time TEXT)"""
        queryDb(run)

    def amendDt(self, dte):
        dt = str(dte).split("-")
        if not correctDt(dt[2]) == dt[2]:
            dt.insert(2, "0" + dt[2])
        return dt[0] + "-" + dt[1] + "-" + dt[2]

    # Main method
    def fetcher(self, link=False, net=True):
        if not link:
            link = "livescore.mobi/football/" + str(self.amendDt(now.date())) + "/?tz=3"
        logging.debug(f"Web-requester Activated : {link}")
        self.createDb()
        if net == True:
            resend = self.sender(link)
        else:
            resend = self.launcher(link)
        return resend



# win,draw,loss evaluater
from .bet_common import getInt, tScore, logging

# Getting performance generally
def GG(results):
    g = gg = ov15 = ov25 = ov35 = df = 0
    dwn = str(results).count("-")
    if dwn:
        df = 100 / dwn
    else:
        df += 0.0000001
        logging.debug("Empty list in tertiary_bet")
        return [g, gg, ov15, ov25, ov35]
    for x in range(3):
        for score in results[x]:
            res = getInt(score)
            sm = sum(res)
            if max(res) and min(res):
                gg += df
            if sm >= 2:
                ov15 += df
            if sm >= 3:
                ov25 += df
            if sm >= 4:
                ov35 += df
            if not x == 2:
                g += max(res)
            else:
                if min(res):
                    g += min(res)
    return [g, gg, ov15, ov25, ov35]


# Getting performance per team level
def GG1(results):
    g = gg = ov15 = ov25 = ov35 = df = 0
    dwn = str(results).count("-")
    if dwn:
        df = 100 / dwn
    else:
        df += 0.000001
        return [g, gg, ov15, ov25, ov35]
    for x in range(3):
        for score in results[x]:
            res = getInt(score)
            if not x == 2:
                sm = max(res)
                g += sm
                if sm:
                    gg += df
                    if sm >= 2:
                        ov15 += df
                    if sm >= 3:
                        ov25 += df
                    if sm >= 4:
                        ov35 += df
            else:
                sm = min(res)
                if sm:
                    g += sm
                    gg += df
                    if sm >= 2:
                        ov15 += df
                    if sm >= 3:
                        ov25 += df
                    if sm >= 4:
                        ov35 += df
    return [g, gg, ov15, ov25, ov35]


def tertiary(wdl):
    No = str(wdl).count("-")
    logging.debug(f"WDL Received [{No}].\n{wdl}")
    G1 = GG(wdl)
    G2 = GG1(wdl)
    t = ["g", "gg", "ov15", "ov25", "ov35"]
    for x in range(1, 5):
        t[x] = round((G1[x] + G2[x]) / 2, 2)
    del wdl
    return [G1[0], t[1], t[2], t[3], t[4]]



#!/usr/bin/python3
import argparse
from . import __version__
from .configuration_handler import set_config
from appdirs import AppDirs

dirs = AppDirs("Smartwa", "smartbets_API")
program_info = "Simple football prediction API  "
epilog = "[*] This program is disseminated under GPL-3.0 license."


def parse_handler():
    parser = argparse.ArgumentParser(description=program_info, epilog=epilog)
    parser.add_argument(
        "-v", "--version", action="version", version="%(prog)s " + "v" + __version__
    )
    parser.add_argument(
        "-u", "--username", help="Username for authenticating clients", default="API"
    )
    parser.add_argument(
        "-p",
        "--port",
        help="Port to be used for hosting the API",
        default=8000,
        type=int,
    )
    parser.add_argument("-f", "--filename", help="Filename to be used in logging")
    parser.add_argument(
        "-l",
        "--level",
        help="Logging level",
        choices=[0, 10, 20, 30, 40, 50],
        type=int,
        default=20,
        metavar="0-50",
    )
    parser.add_argument(
        "--no-net",
        action="store_true",
        dest="no_net",
        help="Force the API to use cached data rather than from internet.",
    )
    parser.add_argument("--host", help="Host the API on net [LAN]", action="store_true")
    parser.add_argument(
        "--debug", help="Debug the API at web level - developers", action="store_true"
    )
    parser.add_argument(
        "--log",
        help=f"Log to a file located at {dirs.user_data_dir}",
        action="store_true",
    )
    parser.add_argument(
        "--colorize", help="Colorize the logs", action="store_true", dest="color"
    )
    parser.add_argument(
        "--gui", help="Use gui notifications [Termux]", action="store_true"
    )
    parser.add_argument("password", help="Passphrase for login authentication")
    return parser.parse_args()


args = parse_handler()
# Ensures configurations are set before importing other modules
set_config(args).main()
from flask import *
from .predictor import predictor
from .bet_common import logging, col
from random import sample

cookie_jar, ip_address = {}, []
app = Flask(__name__)


@app.errorhandler(500)
def internal_error(e):
    return str(e), 500


# Generates cookie value
def cookie_value():
    dat = []
    addr = request.remote_addr
    for x in range(65, 126, 1):
        dat.append(chr(x))
    rp = "".join(sample(dat, 30))
    cookie_jar[addr] = rp
    ip_address.append(addr)
    logging.debug(f"Assigning cookie_value [{rp}] : {request.remote_addr}")
    logging.debug(f"Total logins [{len(cookie_jar)}]")
    return rp


# Interacts with the predictor
def hunter(h: str, a: str, net: bool) -> dict:
    logging.debug(f"BET : {request.remote_addr} : {h}-{a}")
    return jsonify(predictor(api=True).predictorD({1: h, 2: a}, verifyNet(net)))


# Converts net-arg parsed to bool (data-type)
def verifyNet(arg):
    arg = str(arg).lower()
    if arg in ("false", "0", "none"):
        return False
    else:
        return True


# Handles request error
def error(msg: str, comment=""):
    logging.debug(f"INVALID {request.remote_addr} : {msg}")
    return make_response(
        Response(
            f'<script>alert("{msg}")</script>{comment}',
            mimetype="javascript/application",
        )
    )


def bad_request() -> tuple:
    info = """
	Kindly pass team names {home:str,away:str,net:bool}
	//<input name="home" type="text"/> 
	//<input name="away" type="text"/>
	//<input name="net" type="text"/>
	"""
    return error(info), 400


# Route to the server
@app.route("/", methods=["GET", "POST"])
def responser():
    if (
        request.cookies.get("id")
        and request.remote_addr in ip_address
        and cookie_jar[request.remote_addr] == request.cookies.get("id")
    ):
        if request.method == "GET":
            h_team = request.args.get("home")
            a_team = request.args.get("away")
            net = False if args.no_net else request.args.get("net")
            if h_team and a_team:
                return hunter(h_team, a_team, str(net))
            else:
                logging.error(f"Get method unknown : {request.remote_addr}")
                return bad_request()
        else:
            try:
                h_team = request.form["home"]
                a_team = request.form["away"]
                if "net" in list(request.form.keys()):
                    net = False if args.no_net else str(request.form["net"])
            except Exception as e:
                logging.error(f"POST method - {e} - [{request.remote_addr}]")
                return bad_request()
            else:
                return hunter(h_team, a_team, net)
    else:
        return error("Kindly login!"), 401


# Initial login
@app.route("/login", methods=["POST"])
def login():
    try:
        user = request.form["user"]
        paswd = request.form["paswd"]
    except:
        return (
            error(
                "Error in your request",
                '//<input name="user" type="text"/> '
                '//<innput name="paswd" type="password"/>',
            ),
            400,
        )
    if user == args.username and paswd == args.password:
        resp = make_response("<h1>Success<h1>")
        resp.set_cookie("id", cookie_value())
        logging.debug(
            f"LOGIN Successful: {request.remote_addr} : {request.headers['User-Agent']}"
        )
        return resp
    else:
        logging.debug(
            f"LOGIN FAILURE : {request.remote_addr} : {request.headers['User-Agent']}"
        )
        return error("Wrong credentials!"), 400


def start_server():
    try:
        if args.host:
            app.run(host="0.0.0.0", port=args.port, debug=args.debug)
        else:
            app.run(port=args.port, debug=args.debug)
    except (KeyboardInterrupt, EOFError):
        from sys import exit

        exit(logging.critical("KeyboardInterrupt"))
        print(col.Fore.RED + "[*] Goodbye !" + col.Fore.RESET)
    except Exception as e:
        logging.error(str(e))


if __name__ == "__main__":
    start_server()



from setuptools import setup
from smartbets_API import __version__, __author__


def get_file(nm: str) -> str:
    with open(nm) as file:
        return file.readlines()


setup(
    name="smartbetsAPI",
    packages=["smartbets_API"],
    version=__version__,
    install_requires=get_file("requirements.txt"),
    url="https://github.com/Simatwa/smartbetsAPI.git",
    license="GPL-3.0",
    author=__author__,
    author_email="smartwacaleb@gmail.com",
    description="Simple football prediction API",
    long_description="\n".join(get_file("README.md")),
    long_description_content_type="text/markdown",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Operating System :: OS Independent",
    ],
    entry_points={
        "console_scripts": ["smartbetsAPI = smartbets_API.interface:start_server"],
    },
)
